/* SyntheSys system manager header */


typedef struct aout_symbol_table
     {
       unsigned long tabsize;
       unsigned long strsize;
       unsigned long addr;
       unsigned long reserved;
     } aout_symbol_table_t;

typedef struct elf_section_header_table
     {
       unsigned long num;
       unsigned long size;
       unsigned long addr;
       unsigned long shndx;
     } elf_section_header_table_t;

struct SSSysGDTEntry_s
{
    unsigned short LimitLow;
    unsigned short BaseLow;
    unsigned char BaseMiddle;
    unsigned char AccessPrivilieges;
    unsigned char BlockSize;
    unsigned char BaseHigh;
} __attribute__((packed));

struct SSSysGDTPtr_s
{
    unsigned short Limit;
    unsigned int Base;
} __attribute__((packed));

struct SSSysIDTEntry_s
{
    unsigned short BaseLo;
    unsigned short SegmentSelector;        /* Our kernel segment goes here! */
    unsigned char Always0;     /* This will ALWAYS be set to 0! */
    unsigned char AccessFlags;
    unsigned short BaseHi;
} __attribute__((packed));

struct SSSysIDTPtr_s
{
    unsigned short Limit;
    unsigned int Base;
} __attribute__((packed));

struct SSSysRegs_s
{
    unsigned int gs, fs, es, ds;      /* pushed the segs last */
    unsigned int edi, esi, ebp, esp, ebx, edx, ecx, eax;  /* pushed by 'pusha' */
    unsigned int int_no, err_code;    /* our 'push byte #' and ecodes do this */
    unsigned int eip, cs, eflags, useresp, ss;   /* pushed by the processor automatically */ 
};

typedef struct SSMultibootSysInfo_s
{
   unsigned long Flags;
   unsigned long LowerMem;
   unsigned long UpperMem;
   unsigned long BootDevice;
   unsigned long CommandLine;
   unsigned long ModulesCount;
   unsigned long ModulesAddr;
   union u {
      aout_symbol_table_t aout_sym;
      elf_section_header_table_t elf_sec;
    } u;
    unsigned long MemoryMapLength;
    unsigned long MemoryMapAddr;
} SSMultibootSysInfo_s;



extern void SysBoot(unsigned long MultibootMagic, unsigned long MultiBootInfoAddr);
extern void SysHalt(void);
extern void SysGDTUpdate(void);
extern void SysGDTGateSet(int Num, unsigned long Base, unsigned long Limit, unsigned char AccessPrivilieges, unsigned char BlockSize);
extern void SysGDTInstall(void);
extern void SysIDTLoad(void);
extern void SysIDTEntrySet(int Num, unsigned long Base, unsigned short SegmentSelector, unsigned char MaxRingCallable);
extern void SysIDTInstall(void);
extern void SysISRInstall(void);
extern void SysISRHandler(struct SSSysRegs_s *regsp);
extern void SysIRQRemap();
extern void SysIRQHandlerRegister(unsigned char IntNum, void (*IRQHandler)(struct SSSysRegs_s *regsp));
extern void SysIRQHandlerUnregister(unsigned char IntNum);
extern void SysIRQInstall();
extern void SysIRQDispatcher(struct SSSysRegs_s *regsp);

extern void SysISR0();
extern void SysISR1();
extern void SysISR2();
extern void SysISR3();
extern void SysISR3();
extern void SysISR4();
extern void SysISR5();
extern void SysISR6();
extern void SysISR7();
extern void SysISR8();
extern void SysISR9();
extern void SysISR10();
extern void SysISR11();
extern void SysISR12();
extern void SysISR13();
extern void SysISR14();
extern void SysISR15();
extern void SysISR16();
extern void SysISR17();
extern void SysISR18();
extern void SysISR19();
extern void SysISR20();
extern void SysISR21();
extern void SysISR22();
extern void SysISR23();
extern void SysISR24();
extern void SysISR25();
extern void SysISR26();
extern void SysISR27();
extern void SysISR28();
extern void SysISR29();
extern void SysISR30();
extern void SysISR31();

extern void SysIRQ0();
extern void SysIRQ1();
extern void SysIRQ2();
extern void SysIRQ3();
extern void SysIRQ3();
extern void SysIRQ4();
extern void SysIRQ5();
extern void SysIRQ6();
extern void SysIRQ7();
extern void SysIRQ8();
extern void SysIRQ9();
extern void SysIRQ10();
extern void SysIRQ11();
extern void SysIRQ12();
extern void SysIRQ13();
extern void SysIRQ14();
extern void SysIRQ15();

