; This is the kernel entry point. It setups some things before entering the C kernel.
; It setups the stack, the GDT and the IDT of the OS.

[BITS 32]

SECTION .text
global start
start:
	mov esp, _System_stack ; This is a pointer to the system stack
	jmp stublet

ALIGN 4
mboot:
; Multiboot macros to make a few lines later more readable
    MULTIBOOT_PAGE_ALIGN        equ 1<<0
    MULTIBOOT_MEMORY_INFO       equ 1<<1
    MULTIBOOT_AOUT_KLUDGE       equ 1<<16
    MULTIBOOT_HEADER_MAGIC      equ 0x1BADB002
    MULTIBOOT_HEADER_FLAGS      equ MULTIBOOT_PAGE_ALIGN | MULTIBOOT_MEMORY_INFO | MULTIBOOT_AOUT_KLUDGE
    MULTIBOOT_CHECKSUM  equ -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
    EXTERN code, bss, end

    ; This is the GRUB Multiboot header. A boot signature
    dd MULTIBOOT_HEADER_MAGIC
    dd MULTIBOOT_HEADER_FLAGS
    dd MULTIBOOT_CHECKSUM

    ; AOUT kludge - must be physical addresses. Make a note of these:
    ; The linker script fills in the data for these ones!
    dd mboot
    dd code
    dd bss
    dd end
    dd start

stublet:
	;push dword 2 ; Prevents Task switching at IRET
	;popf

	push ebx
	push eax

	extern SysBoot
    	call SysBoot
	jmp $

; SysGDTUpdate: Loads an updated GDT.

global SysGDTUpdate
extern SystemGDT
SysGDTUpdate:

    lgdt [SystemGDT]   ; Loads the GDT defined by the SystemGDT pointer
    mov ax, 0x10      ; 0x10 is the offset in the GDT to our data segment
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov ss, ax
    jmp 0x08:SysGDTUpdate2   ; 0x08 is the offset to our code segment: Far jump!
SysGDTUpdate2:
    ret               ; Returns back to the C code!

global SysIDTLoad
extern SystemIDT
SysIDTLoad:
	lidt [SystemIDT]
	ret

global SysISR0
global SysISR1
global SysISR2
global SysISR3
global SysISR4
global SysISR5
global SysISR6
global SysISR7
global SysISR8
global SysISR9
global SysISR10
global SysISR11
global SysISR12
global SysISR13
global SysISR14
global SysISR15
global SysISR16
global SysISR17
global SysISR18
global SysISR19
global SysISR20
global SysISR21
global SysISR22
global SysISR23
global SysISR24
global SysISR25
global SysISR26
global SysISR27
global SysISR28
global SysISR29
global SysISR30
global SysISR31


SysISR0:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 0
    jmp SysISRCommonCode

SysISR1:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 1
    jmp SysISRCommonCode

SysISR2:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 2
    jmp SysISRCommonCode

SysISR3:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 3
    jmp SysISRCommonCode

SysISR4:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 4
    jmp SysISRCommonCode

SysISR5:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 5
    jmp SysISRCommonCode

SysISR6:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 6
    jmp SysISRCommonCode

SysISR7:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 7
    jmp SysISRCommonCode

SysISR8:
    cli
		; We do not need to push something on the stack as the exception does it.
    push byte 8
    jmp SysISRCommonCode

SysISR9:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 9
    jmp SysISRCommonCode

SysISR10:
    cli
    push byte 10
    jmp SysISRCommonCode

SysISR11:
    cli
		; We do not need to push something on the stack as the exception does it.
    push byte 11
    jmp SysISRCommonCode

SysISR12:
    cli
		; We do not need to push something on the stack as the exception does it.
    push byte 12
    jmp SysISRCommonCode

SysISR13:
    cli
		; We do not need to push something on the stack as the exception does it.
    push byte 13
    jmp SysISRCommonCode

SysISR14:
    cli
		; We do not need to push something on the stack as the exception does it.
    push byte 14
    jmp SysISRCommonCode

SysISR15:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 15
    jmp SysISRCommonCode

SysISR16:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 16
    jmp SysISRCommonCode

SysISR17:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 17
    jmp SysISRCommonCode

SysISR18:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 18
    jmp SysISRCommonCode

SysISR19:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 19
    jmp SysISRCommonCode

SysISR20:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 20
    jmp SysISRCommonCode

SysISR21:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 21
    jmp SysISRCommonCode

SysISR22:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 22
    jmp SysISRCommonCode

SysISR23:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 23
    jmp SysISRCommonCode

SysISR24:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 24
    jmp SysISRCommonCode

SysISR25:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 25
    jmp SysISRCommonCode

SysISR26:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 26
    jmp SysISRCommonCode

SysISR27:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 27
    jmp SysISRCommonCode

SysISR28:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 28
    jmp SysISRCommonCode

SysISR29:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 29
    jmp SysISRCommonCode

SysISR30:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 30
    jmp SysISRCommonCode

SysISR31:
    cli
    push byte 0    ; A normal ISR stub that pops a dummy error code to keep a
                   ; uniform stack frame
    push byte 31
    jmp SysISRCommonCode

extern SysISRHandler
SysISRCommonCode:
    pusha
    push ds
    push es
    push fs
    push gs
    mov ax, 0x10   ; Load the Kernel Data Segment descriptor!
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov eax, esp   ; Push us the stack
    push eax
    mov eax, SysISRHandler
    call eax       ; A special call, preserves the 'eip' register
    pop eax
    pop gs
    pop fs
    pop es
    pop ds
    popa
    add esp, 8     ; Cleans up the pushed error code and pushed ISR number
    iret           ; pops 5 things at once: CS, EIP, EFLAGS, SS, and ESP!

global SysIRQ0
global SysIRQ1
global SysIRQ2
global SysIRQ3
global SysIRQ4
global SysIRQ5
global SysIRQ6
global SysIRQ7
global SysIRQ8
global SysIRQ9
global SysIRQ10
global SysIRQ11
global SysIRQ12
global SysIRQ13
global SysIRQ14
global SysIRQ15

SysIRQ0:
    cli
    push byte 0    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ1:
    cli
    push byte 1    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ2:
    cli
    push byte 2    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ3:
    cli
    push byte 3    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ4:
    cli
    push byte 4    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ5:
    cli
    push byte 5    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ6:
    cli
    push byte 6    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ7:
    cli
    push byte 7    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ8:
    cli
    push byte 8    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ9:
    cli
    push byte 9    ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ10:
    cli
    push byte 10   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ11:
    cli
    push byte 11   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ12:
    cli
    push byte 12   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ13:
    cli
    push byte 13   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ14:
    cli
    push byte 14   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode

SysIRQ15:
    cli
    push byte 15   ; Note that these don't push an error code on the stack:
                   ; We need to push a dummy error code
    push byte 32
    jmp SysIRQCommonCode


extern SysIRQDispatcher
SysIRQCommonCode:
    pusha
    push ds
    push es
    push fs
    push gs
    mov ax, 0x10
    mov ds, ax
    mov es, ax
    mov fs, ax
    mov gs, ax
    mov eax, esp
    push eax
    mov eax, SysIRQDispatcher
    call eax
    pop eax
    pop gs
    pop fs
    pop es
    pop ds
    popa
    add esp, 8
    iret

SECTION .bss
    resb 8192               ; This reserves 8KBytes of memory here
_System_stack: